"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file

Copyright 2025 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import collections.abc
import dapr.proto.common.v1.common_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class RegisterActorTimerRequest(google.protobuf.message.Message):
    """RegisterActorTimerRequest is the message to register a timer for an actor of a given type and id."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DUE_TIME_FIELD_NUMBER: builtins.int
    PERIOD_FIELD_NUMBER: builtins.int
    CALLBACK_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    TTL_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    name: builtins.str
    due_time: builtins.str
    period: builtins.str
    callback: builtins.str
    data: builtins.bytes
    ttl: builtins.str
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        name: builtins.str = ...,
        due_time: builtins.str = ...,
        period: builtins.str = ...,
        callback: builtins.str = ...,
        data: builtins.bytes = ...,
        ttl: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["actor_id", b"actor_id", "actor_type", b"actor_type", "callback", b"callback", "data", b"data", "due_time", b"due_time", "name", b"name", "period", b"period", "ttl", b"ttl"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RegisterActorTimerRequest: typing_extensions.TypeAlias = RegisterActorTimerRequest

@typing.final
class UnregisterActorTimerRequest(google.protobuf.message.Message):
    """UnregisterActorTimerRequest is the message to unregister an actor timer"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    name: builtins.str
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["actor_id", b"actor_id", "actor_type", b"actor_type", "name", b"name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___UnregisterActorTimerRequest: typing_extensions.TypeAlias = UnregisterActorTimerRequest

@typing.final
class RegisterActorReminderRequest(google.protobuf.message.Message):
    """RegisterActorReminderRequest is the message to register a reminder for an actor of a given type and id."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DUE_TIME_FIELD_NUMBER: builtins.int
    PERIOD_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    TTL_FIELD_NUMBER: builtins.int
    OVERWRITE_FIELD_NUMBER: builtins.int
    FAILURE_POLICY_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    name: builtins.str
    due_time: builtins.str
    period: builtins.str
    data: builtins.bytes
    ttl: builtins.str
    overwrite: builtins.bool
    """If true, allows this reminder to overwrite an existing reminder with the
    same name. If not set, defaults to true.
    """
    @property
    def failure_policy(self) -> dapr.proto.common.v1.common_pb2.JobFailurePolicy:
        """failure_policy is the optional policy for handling job failures. If not
        set, the reminder will have the failure policy of trying 3 times on a
        single tick before giving up.
        """

    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        name: builtins.str = ...,
        due_time: builtins.str = ...,
        period: builtins.str = ...,
        data: builtins.bytes = ...,
        ttl: builtins.str = ...,
        overwrite: builtins.bool | None = ...,
        failure_policy: dapr.proto.common.v1.common_pb2.JobFailurePolicy | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["_failure_policy", b"_failure_policy", "_overwrite", b"_overwrite", "failure_policy", b"failure_policy", "overwrite", b"overwrite"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["_failure_policy", b"_failure_policy", "_overwrite", b"_overwrite", "actor_id", b"actor_id", "actor_type", b"actor_type", "data", b"data", "due_time", b"due_time", "failure_policy", b"failure_policy", "name", b"name", "overwrite", b"overwrite", "period", b"period", "ttl", b"ttl"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__failure_policy: typing_extensions.TypeAlias = typing.Literal["failure_policy"]
    _WhichOneofArgType__failure_policy: typing_extensions.TypeAlias = typing.Literal["_failure_policy", b"_failure_policy"]
    _WhichOneofReturnType__overwrite: typing_extensions.TypeAlias = typing.Literal["overwrite"]
    _WhichOneofArgType__overwrite: typing_extensions.TypeAlias = typing.Literal["_overwrite", b"_overwrite"]
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__failure_policy) -> _WhichOneofReturnType__failure_policy | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__overwrite) -> _WhichOneofReturnType__overwrite | None: ...

Global___RegisterActorReminderRequest: typing_extensions.TypeAlias = RegisterActorReminderRequest

@typing.final
class UnregisterActorReminderRequest(google.protobuf.message.Message):
    """UnregisterActorReminderRequest is the message to unregister an actor reminder."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    name: builtins.str
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["actor_id", b"actor_id", "actor_type", b"actor_type", "name", b"name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___UnregisterActorReminderRequest: typing_extensions.TypeAlias = UnregisterActorReminderRequest

@typing.final
class GetActorStateRequest(google.protobuf.message.Message):
    """GetActorStateRequest is the message to get key-value states from specific actor."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    key: builtins.str
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        key: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["actor_id", b"actor_id", "actor_type", b"actor_type", "key", b"key"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GetActorStateRequest: typing_extensions.TypeAlias = GetActorStateRequest

@typing.final
class GetActorStateResponse(google.protobuf.message.Message):
    """GetActorStateResponse is the response conveying the actor's state value."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    DATA_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to app."""

    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["data", b"data", "metadata", b"metadata"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GetActorStateResponse: typing_extensions.TypeAlias = GetActorStateResponse

@typing.final
class ExecuteActorStateTransactionRequest(google.protobuf.message.Message):
    """ExecuteActorStateTransactionRequest is the message to execute multiple operations on a specified actor."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    OPERATIONS_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    @property
    def operations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___TransactionalActorStateOperation]: ...
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        operations: collections.abc.Iterable[Global___TransactionalActorStateOperation] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["actor_id", b"actor_id", "actor_type", b"actor_type", "operations", b"operations"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ExecuteActorStateTransactionRequest: typing_extensions.TypeAlias = ExecuteActorStateTransactionRequest

@typing.final
class TransactionalActorStateOperation(google.protobuf.message.Message):
    """TransactionalActorStateOperation is the message to execute a specified operation with a key-value pair."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    OPERATIONTYPE_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    operationType: builtins.str
    key: builtins.str
    @property
    def value(self) -> google.protobuf.any_pb2.Any: ...
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata used for transactional operations.

        Common metadata property:
        - ttlInSeconds : the time to live in seconds for the stored value.
        """

    def __init__(
        self,
        *,
        operationType: builtins.str = ...,
        key: builtins.str = ...,
        value: google.protobuf.any_pb2.Any | None = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["value", b"value"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "metadata", b"metadata", "operationType", b"operationType", "value", b"value"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___TransactionalActorStateOperation: typing_extensions.TypeAlias = TransactionalActorStateOperation

@typing.final
class InvokeActorRequest(google.protobuf.message.Message):
    """InvokeActorRequest is the message to call an actor."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    METHOD_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    method: builtins.str
    data: builtins.bytes
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        method: builtins.str = ...,
        data: builtins.bytes = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["actor_id", b"actor_id", "actor_type", b"actor_type", "data", b"data", "metadata", b"metadata", "method", b"method"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___InvokeActorRequest: typing_extensions.TypeAlias = InvokeActorRequest

@typing.final
class InvokeActorResponse(google.protobuf.message.Message):
    """InvokeActorResponse is the method that returns an actor invocation response."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["data", b"data"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___InvokeActorResponse: typing_extensions.TypeAlias = InvokeActorResponse

@typing.final
class GetActorReminderRequest(google.protobuf.message.Message):
    """GetActorReminderRequest is the message to get an already-registered actor reminder"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    name: builtins.str
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["actor_id", b"actor_id", "actor_type", b"actor_type", "name", b"name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GetActorReminderRequest: typing_extensions.TypeAlias = GetActorReminderRequest

@typing.final
class GetActorReminderResponse(google.protobuf.message.Message):
    """GetActorReminderResponse is the response conveying an actor's reminder."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    DUE_TIME_FIELD_NUMBER: builtins.int
    PERIOD_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    TTL_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    due_time: builtins.str
    period: builtins.str
    ttl: builtins.str
    @property
    def data(self) -> google.protobuf.any_pb2.Any: ...
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        due_time: builtins.str | None = ...,
        period: builtins.str | None = ...,
        data: google.protobuf.any_pb2.Any | None = ...,
        ttl: builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["_due_time", b"_due_time", "_period", b"_period", "_ttl", b"_ttl", "data", b"data", "due_time", b"due_time", "period", b"period", "ttl", b"ttl"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["_due_time", b"_due_time", "_period", b"_period", "_ttl", b"_ttl", "actor_id", b"actor_id", "actor_type", b"actor_type", "data", b"data", "due_time", b"due_time", "period", b"period", "ttl", b"ttl"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__due_time: typing_extensions.TypeAlias = typing.Literal["due_time"]
    _WhichOneofArgType__due_time: typing_extensions.TypeAlias = typing.Literal["_due_time", b"_due_time"]
    _WhichOneofReturnType__period: typing_extensions.TypeAlias = typing.Literal["period"]
    _WhichOneofArgType__period: typing_extensions.TypeAlias = typing.Literal["_period", b"_period"]
    _WhichOneofReturnType__ttl: typing_extensions.TypeAlias = typing.Literal["ttl"]
    _WhichOneofArgType__ttl: typing_extensions.TypeAlias = typing.Literal["_ttl", b"_ttl"]
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__due_time) -> _WhichOneofReturnType__due_time | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__period) -> _WhichOneofReturnType__period | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__ttl) -> _WhichOneofReturnType__ttl | None: ...

Global___GetActorReminderResponse: typing_extensions.TypeAlias = GetActorReminderResponse

@typing.final
class ListActorRemindersRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["_actor_id", b"_actor_id", "actor_id", b"actor_id"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["_actor_id", b"_actor_id", "actor_id", b"actor_id", "actor_type", b"actor_type"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__actor_id: typing_extensions.TypeAlias = typing.Literal["actor_id"]
    _WhichOneofArgType__actor_id: typing_extensions.TypeAlias = typing.Literal["_actor_id", b"_actor_id"]
    def WhichOneof(self, oneof_group: _WhichOneofArgType__actor_id) -> _WhichOneofReturnType__actor_id | None: ...

Global___ListActorRemindersRequest: typing_extensions.TypeAlias = ListActorRemindersRequest

@typing.final
class ListActorRemindersResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REMINDERS_FIELD_NUMBER: builtins.int
    @property
    def reminders(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___NamedActorReminder]: ...
    def __init__(
        self,
        *,
        reminders: collections.abc.Iterable[Global___NamedActorReminder] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["reminders", b"reminders"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ListActorRemindersResponse: typing_extensions.TypeAlias = ListActorRemindersResponse

@typing.final
class NamedActorReminder(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    REMINDER_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def reminder(self) -> Global___ActorReminder: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        reminder: Global___ActorReminder | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["reminder", b"reminder"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["name", b"name", "reminder", b"reminder"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___NamedActorReminder: typing_extensions.TypeAlias = NamedActorReminder

@typing.final
class ActorReminder(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    DUE_TIME_FIELD_NUMBER: builtins.int
    PERIOD_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    TTL_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    due_time: builtins.str
    period: builtins.str
    ttl: builtins.str
    @property
    def data(self) -> google.protobuf.any_pb2.Any: ...
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str = ...,
        due_time: builtins.str | None = ...,
        period: builtins.str | None = ...,
        data: google.protobuf.any_pb2.Any | None = ...,
        ttl: builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["_due_time", b"_due_time", "_period", b"_period", "_ttl", b"_ttl", "data", b"data", "due_time", b"due_time", "period", b"period", "ttl", b"ttl"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["_due_time", b"_due_time", "_period", b"_period", "_ttl", b"_ttl", "actor_id", b"actor_id", "actor_type", b"actor_type", "data", b"data", "due_time", b"due_time", "period", b"period", "ttl", b"ttl"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__due_time: typing_extensions.TypeAlias = typing.Literal["due_time"]
    _WhichOneofArgType__due_time: typing_extensions.TypeAlias = typing.Literal["_due_time", b"_due_time"]
    _WhichOneofReturnType__period: typing_extensions.TypeAlias = typing.Literal["period"]
    _WhichOneofArgType__period: typing_extensions.TypeAlias = typing.Literal["_period", b"_period"]
    _WhichOneofReturnType__ttl: typing_extensions.TypeAlias = typing.Literal["ttl"]
    _WhichOneofArgType__ttl: typing_extensions.TypeAlias = typing.Literal["_ttl", b"_ttl"]
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__due_time) -> _WhichOneofReturnType__due_time | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__period) -> _WhichOneofReturnType__period | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__ttl) -> _WhichOneofReturnType__ttl | None: ...

Global___ActorReminder: typing_extensions.TypeAlias = ActorReminder

@typing.final
class UnregisterActorRemindersByTypeRequest(google.protobuf.message.Message):
    """UnregisterActorRemindersByTypeRequest is the message to unregister an actor
    reminders by the given type. Optional actor_id can be provided to limit the
    scope of the operation to a specific actor instance.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTOR_TYPE_FIELD_NUMBER: builtins.int
    ACTOR_ID_FIELD_NUMBER: builtins.int
    actor_type: builtins.str
    actor_id: builtins.str
    def __init__(
        self,
        *,
        actor_type: builtins.str = ...,
        actor_id: builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["_actor_id", b"_actor_id", "actor_id", b"actor_id"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["_actor_id", b"_actor_id", "actor_id", b"actor_id", "actor_type", b"actor_type"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__actor_id: typing_extensions.TypeAlias = typing.Literal["actor_id"]
    _WhichOneofArgType__actor_id: typing_extensions.TypeAlias = typing.Literal["_actor_id", b"_actor_id"]
    def WhichOneof(self, oneof_group: _WhichOneofArgType__actor_id) -> _WhichOneofReturnType__actor_id | None: ...

Global___UnregisterActorRemindersByTypeRequest: typing_extensions.TypeAlias = UnregisterActorRemindersByTypeRequest

@typing.final
class UnregisterActorRemindersByTypeResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___UnregisterActorRemindersByTypeResponse: typing_extensions.TypeAlias = UnregisterActorRemindersByTypeResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: dapr/proto/runtime/v1/workflow.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'dapr/proto/runtime/v1/workflow.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$dapr/proto/runtime/v1/workflow.proto\x12\x15\x64\x61pr.proto.runtime.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"d\n\x12GetWorkflowRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12-\n\x12workflow_component\x18\x02 \x01(\tR\x11workflowComponent\"\x84\x03\n\x13GetWorkflowResponse\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12#\n\rworkflow_name\x18\x02 \x01(\tR\x0cworkflowName\x12\x39\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x42\n\x0flast_updated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rlastUpdatedAt\x12%\n\x0eruntime_status\x18\x05 \x01(\tR\rruntimeStatus\x12N\n\nproperties\x18\x06 \x03(\x0b\x32:.dapr.proto.runtime.v1.GetWorkflowResponse.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x95\x02\n\x14StartWorkflowRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12-\n\x12workflow_component\x18\x02 \x01(\tR\x11workflowComponent\x12#\n\rworkflow_name\x18\x03 \x01(\tR\x0cworkflowName\x12I\n\x07options\x18\x04 \x03(\x0b\x32\x38.dapr.proto.runtime.v1.StartWorkflowRequest.OptionsEntry\x12\r\n\x05input\x18\x05 \x01(\x0c\x1a.\n\x0cOptionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"8\n\x15StartWorkflowResponse\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\"j\n\x18TerminateWorkflowRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12-\n\x12workflow_component\x18\x02 \x01(\tR\x11workflowComponent\"f\n\x14PauseWorkflowRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12-\n\x12workflow_component\x18\x02 \x01(\tR\x11workflowComponent\"g\n\x15ResumeWorkflowRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12-\n\x12workflow_component\x18\x02 \x01(\tR\x11workflowComponent\"\x9e\x01\n\x19RaiseEventWorkflowRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12-\n\x12workflow_component\x18\x02 \x01(\tR\x11workflowComponent\x12\x1d\n\nevent_name\x18\x03 \x01(\tR\teventName\x12\x12\n\nevent_data\x18\x04 \x01(\x0c\"f\n\x14PurgeWorkflowRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceID\x12-\n\x12workflow_component\x18\x02 \x01(\tR\x11workflowComponentBi\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\xaa\x02\x1b\x44\x61pr.Client.Autogen.Grpc.v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dapr.proto.runtime.v1.workflow_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\252\002\033Dapr.Client.Autogen.Grpc.v1'
  _globals['_GETWORKFLOWRESPONSE_PROPERTIESENTRY']._loaded_options = None
  _globals['_GETWORKFLOWRESPONSE_PROPERTIESENTRY']._serialized_options = b'8\001'
  _globals['_STARTWORKFLOWREQUEST_OPTIONSENTRY']._loaded_options = None
  _globals['_STARTWORKFLOWREQUEST_OPTIONSENTRY']._serialized_options = b'8\001'
  _globals['_GETWORKFLOWREQUEST']._serialized_start=96
  _globals['_GETWORKFLOWREQUEST']._serialized_end=196
  _globals['_GETWORKFLOWRESPONSE']._serialized_start=199
  _globals['_GETWORKFLOWRESPONSE']._serialized_end=587
  _globals['_GETWORKFLOWRESPONSE_PROPERTIESENTRY']._serialized_start=538
  _globals['_GETWORKFLOWRESPONSE_PROPERTIESENTRY']._serialized_end=587
  _globals['_STARTWORKFLOWREQUEST']._serialized_start=590
  _globals['_STARTWORKFLOWREQUEST']._serialized_end=867
  _globals['_STARTWORKFLOWREQUEST_OPTIONSENTRY']._serialized_start=821
  _globals['_STARTWORKFLOWREQUEST_OPTIONSENTRY']._serialized_end=867
  _globals['_STARTWORKFLOWRESPONSE']._serialized_start=869
  _globals['_STARTWORKFLOWRESPONSE']._serialized_end=925
  _globals['_TERMINATEWORKFLOWREQUEST']._serialized_start=927
  _globals['_TERMINATEWORKFLOWREQUEST']._serialized_end=1033
  _globals['_PAUSEWORKFLOWREQUEST']._serialized_start=1035
  _globals['_PAUSEWORKFLOWREQUEST']._serialized_end=1137
  _globals['_RESUMEWORKFLOWREQUEST']._serialized_start=1139
  _globals['_RESUMEWORKFLOWREQUEST']._serialized_end=1242
  _globals['_RAISEEVENTWORKFLOWREQUEST']._serialized_start=1245
  _globals['_RAISEEVENTWORKFLOWREQUEST']._serialized_end=1403
  _globals['_PURGEWORKFLOWREQUEST']._serialized_start=1405
  _globals['_PURGEWORKFLOWREQUEST']._serialized_end=1507
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: dapr/proto/runtime/v1/secret.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'dapr/proto/runtime/v1/secret.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"dapr/proto/runtime/v1/secret.proto\x12\x15\x64\x61pr.proto.runtime.v1\"\xb8\x01\n\x10GetSecretRequest\x12\x1d\n\nstore_name\x18\x01 \x01(\tR\tstoreName\x12\x0b\n\x03key\x18\x02 \x01(\t\x12G\n\x08metadata\x18\x03 \x03(\x0b\x32\x35.dapr.proto.runtime.v1.GetSecretRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x82\x01\n\x11GetSecretResponse\x12@\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x32.dapr.proto.runtime.v1.GetSecretResponse.DataEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb3\x01\n\x14GetBulkSecretRequest\x12\x1d\n\nstore_name\x18\x01 \x01(\tR\tstoreName\x12K\n\x08metadata\x18\x02 \x03(\x0b\x32\x39.dapr.proto.runtime.v1.GetBulkSecretRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x85\x01\n\x0eSecretResponse\x12\x43\n\x07secrets\x18\x01 \x03(\x0b\x32\x32.dapr.proto.runtime.v1.SecretResponse.SecretsEntry\x1a.\n\x0cSecretsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb1\x01\n\x15GetBulkSecretResponse\x12\x44\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x36.dapr.proto.runtime.v1.GetBulkSecretResponse.DataEntry\x1aR\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.dapr.proto.runtime.v1.SecretResponse:\x02\x38\x01\x42i\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\xaa\x02\x1b\x44\x61pr.Client.Autogen.Grpc.v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dapr.proto.runtime.v1.secret_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\252\002\033Dapr.Client.Autogen.Grpc.v1'
  _globals['_GETSECRETREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_GETSECRETREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_GETSECRETRESPONSE_DATAENTRY']._loaded_options = None
  _globals['_GETSECRETRESPONSE_DATAENTRY']._serialized_options = b'8\001'
  _globals['_GETBULKSECRETREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_GETBULKSECRETREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_SECRETRESPONSE_SECRETSENTRY']._loaded_options = None
  _globals['_SECRETRESPONSE_SECRETSENTRY']._serialized_options = b'8\001'
  _globals['_GETBULKSECRETRESPONSE_DATAENTRY']._loaded_options = None
  _globals['_GETBULKSECRETRESPONSE_DATAENTRY']._serialized_options = b'8\001'
  _globals['_GETSECRETREQUEST']._serialized_start=62
  _globals['_GETSECRETREQUEST']._serialized_end=246
  _globals['_GETSECRETREQUEST_METADATAENTRY']._serialized_start=199
  _globals['_GETSECRETREQUEST_METADATAENTRY']._serialized_end=246
  _globals['_GETSECRETRESPONSE']._serialized_start=249
  _globals['_GETSECRETRESPONSE']._serialized_end=379
  _globals['_GETSECRETRESPONSE_DATAENTRY']._serialized_start=336
  _globals['_GETSECRETRESPONSE_DATAENTRY']._serialized_end=379
  _globals['_GETBULKSECRETREQUEST']._serialized_start=382
  _globals['_GETBULKSECRETREQUEST']._serialized_end=561
  _globals['_GETBULKSECRETREQUEST_METADATAENTRY']._serialized_start=199
  _globals['_GETBULKSECRETREQUEST_METADATAENTRY']._serialized_end=246
  _globals['_SECRETRESPONSE']._serialized_start=564
  _globals['_SECRETRESPONSE']._serialized_end=697
  _globals['_SECRETRESPONSE_SECRETSENTRY']._serialized_start=651
  _globals['_SECRETRESPONSE_SECRETSENTRY']._serialized_end=697
  _globals['_GETBULKSECRETRESPONSE']._serialized_start=700
  _globals['_GETBULKSECRETRESPONSE']._serialized_end=877
  _globals['_GETBULKSECRETRESPONSE_DATAENTRY']._serialized_start=795
  _globals['_GETBULKSECRETRESPONSE_DATAENTRY']._serialized_end=877
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

"""
Copyright 2023 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import Enum
from typing import Any, Dict, List, Optional, Tuple, Union

from google.protobuf import json_format
from google.protobuf.any_pb2 import Any as GrpcAny
from google.protobuf.message import Message as GrpcMessage
from google.protobuf.struct_pb2 import Struct
from google.protobuf.wrappers_pb2 import (
    BoolValue,
    BytesValue,
    DoubleValue,
    Int32Value,
    Int64Value,
    StringValue,
)

MetadataDict = Dict[str, List[Union[bytes, str]]]
MetadataTuple = Tuple[Tuple[str, Union[bytes, str]], ...]


def tuple_to_dict(tupledata: MetadataTuple) -> MetadataDict:
    """Converts tuple to dict.

    Args:
        tupledata (tuple): tuple storing metadata

    Returns:
        A dict which is converted from tuple
    """

    d: MetadataDict = {}
    for k, v in tupledata:  # type: ignore
        d.setdefault(k, []).append(v)
    return d


def unpack(data: GrpcAny, message: GrpcMessage) -> None:
    """Unpack the serialized protocol buffer message.

    Args:
        data (:obj:`google.protobuf.message.Any`): the serialized protocol buffer message.
        message (:obj:`google.protobuf.message.Message`): the protocol buffer message object
            to which the response data is deserialized.

    Raises:
        ValueError: message is not protocol buffer message object or message's type is not
            matched with the response data type
    """
    if not isinstance(message, GrpcMessage):
        raise ValueError('output message is not protocol buffer message object')
    if not data.Is(message.DESCRIPTOR):
        raise ValueError(f'invalid type. serialized message type: {data.type_url}')
    data.Unpack(message)


def to_bytes(data: Union[str, bytes]) -> bytes:
    """Convert str data to bytes."""
    if isinstance(data, bytes):
        return data
    elif isinstance(data, str):
        return data.encode('utf-8')
    else:
        raise f'invalid data type {type(data)}'


def to_str(data: Union[str, bytes]) -> str:
    """Convert bytes data to str."""
    if isinstance(data, str):
        return data
    elif isinstance(data, bytes):
        return data.decode('utf-8')
    else:
        raise f'invalid data type {type(data)}'


# Data validation helpers
def validateNotNone(**kwargs: Optional[str]):
    for field_name, value in kwargs.items():
        if value is None:
            raise ValueError(f'{field_name} name cannot be None')


def validateNotBlankString(**kwargs: Optional[str]):
    for field_name, value in kwargs.items():
        if not value or not value.strip():
            raise ValueError(f'{field_name} name cannot be empty or blank')


class WorkflowRuntimeStatus(Enum):
    UNKNOWN = 'Unknown'
    RUNNING = 'Running'
    COMPLETED = 'Completed'
    FAILED = 'Failed'
    TERMINATED = 'Terminated'
    PENDING = 'Pending'
    SUSPENDED = 'Suspended'


# Will return the enum entry if it is present, otherwise returns "unknown"
def getWorkflowRuntimeStatus(inputString):
    try:
        return WorkflowRuntimeStatus[inputString].value
    except KeyError:
        return WorkflowRuntimeStatus.UNKNOWN


def convert_value_to_struct(value: Dict[str, Any]) -> Struct:
    """Convert a raw Python value to a protobuf Struct message.

    This function converts Python values to a protobuf Struct, which is designed
    to represent JSON-like dynamic data structures.

    Args:
        value: Raw Python value (str, int, float, bool, None, dict, list, or already Struct)

    Returns:
        Struct: The value converted to a protobuf Struct message

    Raises:
        ValueError: If the value type is not supported or cannot be serialized

    Examples:
        >>> convert_value_to_struct("hello")  # -> Struct with string value
        >>> convert_value_to_struct(42)       # -> Struct with number value
        >>> convert_value_to_struct(True)     # -> Struct with bool value
        >>> convert_value_to_struct({"key": "value"})  # -> Struct with nested structure
    """
    # If it's already a Struct, return as-is (backward compatibility)
    if isinstance(value, Struct):
        return value

    # raise an error if the value is not a dictionary
    if not isinstance(value, dict) and not isinstance(value, bytes):
        raise ValueError(f'Value must be a dictionary, got {type(value)}')

    # Convert the value to a JSON-serializable format first
    # Handle bytes by converting to base64 string for JSON compatibility
    if isinstance(value, bytes):
        import base64

        json_value = base64.b64encode(value).decode('utf-8')
    else:
        json_value = value

    try:
        # For dict values, use ParseDict directly
        struct = Struct()
        json_format.ParseDict(json_value, struct)
        return struct

    except (TypeError, ValueError) as e:
        raise ValueError(
            f'Unsupported parameter type or value: {type(value)} = {repr(value)}. '
            f'Must be JSON-serializable. Error: {e}'
        ) from e


def convert_value_to_grpc_any(value: Any) -> GrpcAny:
    """Convert a raw Python value to a GrpcAny protobuf message.
    This function automatically detects the type of the input value and wraps it
    in the appropriate protobuf wrapper type before packing it into GrpcAny.
    Args:
        value: Raw Python value (str, int, float, bool, bytes, or already GrpcAny)
    Returns:
        GrpcAny: The value wrapped in a GrpcAny protobuf message
    Raises:
        ValueError: If the value type is not supported
    Examples:
        >>> convert_value_to_grpc_any("hello")  # -> GrpcAny containing StringValue
        >>> convert_value_to_grpc_any(42)       # -> GrpcAny containing Int64Value
        >>> convert_value_to_grpc_any(3.14)     # -> GrpcAny containing DoubleValue
        >>> convert_value_to_grpc_any(True)     # -> GrpcAny containing BoolValue
    """
    # If it's already a GrpcAny, return as-is (backward compatibility)
    if isinstance(value, GrpcAny):
        return value

    # Create the GrpcAny wrapper
    any_pb = GrpcAny()

    # Convert based on Python type
    if isinstance(value, bool):
        # Note: bool check must come before int since bool is a subclass of int in Python
        any_pb.Pack(BoolValue(value=value))
    elif isinstance(value, str):
        any_pb.Pack(StringValue(value=value))
    elif isinstance(value, int):
        # Use Int64Value to handle larger integers, but Int32Value for smaller ones
        if -2147483648 <= value <= 2147483647:
            any_pb.Pack(Int32Value(value=value))
        else:
            any_pb.Pack(Int64Value(value=value))
    elif isinstance(value, float):
        any_pb.Pack(DoubleValue(value=value))
    elif isinstance(value, bytes):
        any_pb.Pack(BytesValue(value=value))
    else:
        raise ValueError(
            f'Unsupported parameter type: {type(value)}. '
            f'Supported types: str, int, float, bool, bytes, GrpcAny'
        )

    return any_pb


def convert_dict_to_grpc_dict_of_any(parameters: Optional[Dict[str, Any]]) -> Dict[str, GrpcAny]:
    """Convert a dictionary of raw Python values to GrpcAny parameters.
    This function takes a dictionary with raw Python values and converts each
    value to the appropriate GrpcAny protobuf message for use in Dapr API calls.
    Args:
        parameters: Optional dictionary of parameter names to raw Python values
    Returns:
        Dictionary of parameter names to GrpcAny values
    Examples:
        >>> convert_dict_to_grpc_dict_of_any({"temperature": 0.7, "max_tokens": 1000, "stream": False})
        >>> # Returns: {"temperature": GrpcAny, "max_tokens": GrpcAny, "stream": GrpcAny}
    """
    if not parameters:
        return {}

    converted = {}
    for key, value in parameters.items():
        converted[key] = convert_value_to_grpc_any(value)

    return converted



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: dapr/proto/runtime/v1/actors.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'dapr/proto/runtime/v1/actors.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from dapr.proto.common.v1 import common_pb2 as dapr_dot_proto_dot_common_dot_v1_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"dapr/proto/runtime/v1/actors.proto\x12\x15\x64\x61pr.proto.runtime.v1\x1a\x19google/protobuf/any.proto\x1a!dapr/proto/common/v1/common.proto\"\xbb\x01\n\x19RegisterActorTimerRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torId\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x19\n\x08\x64ue_time\x18\x04 \x01(\tR\x07\x64ueTime\x12\x0e\n\x06period\x18\x05 \x01(\t\x12\x10\n\x08\x63\x61llback\x18\x06 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x07 \x01(\x0c\x12\x0b\n\x03ttl\x18\x08 \x01(\t\"e\n\x1bUnregisterActorTimerRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torId\x12\x0c\n\x04name\x18\x03 \x01(\t\"\xba\x02\n\x1cRegisterActorReminderRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torId\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x19\n\x08\x64ue_time\x18\x04 \x01(\tR\x07\x64ueTime\x12\x0e\n\x06period\x18\x05 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\x12\x0b\n\x03ttl\x18\x07 \x01(\t\x12\x16\n\toverwrite\x18\x08 \x01(\x08H\x00\x88\x01\x01\x12S\n\x0e\x66\x61ilure_policy\x18\t \x01(\x0b\x32&.dapr.proto.common.v1.JobFailurePolicyH\x01R\x0e\x66\x61ilure_policy\x88\x01\x01\x42\x0c\n\n_overwriteB\x11\n\x0f_failure_policy\"h\n\x1eUnregisterActorReminderRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torId\x12\x0c\n\x04name\x18\x03 \x01(\t\"]\n\x14GetActorStateRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torId\x12\x0b\n\x03key\x18\x03 \x01(\t\"\xa4\x01\n\x15GetActorStateResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12L\n\x08metadata\x18\x02 \x03(\x0b\x32:.dapr.proto.runtime.v1.GetActorStateResponse.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xac\x01\n#ExecuteActorStateTransactionRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torId\x12K\n\noperations\x18\x03 \x03(\x0b\x32\x37.dapr.proto.runtime.v1.TransactionalActorStateOperation\"\xf5\x01\n TransactionalActorStateOperation\x12\x15\n\roperationType\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12#\n\x05value\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\x12W\n\x08metadata\x18\x04 \x03(\x0b\x32\x45.dapr.proto.runtime.v1.TransactionalActorStateOperation.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xe8\x01\n\x12InvokeActorRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torId\x12\x0e\n\x06method\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\x12I\n\x08metadata\x18\x05 \x03(\x0b\x32\x37.dapr.proto.runtime.v1.InvokeActorRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"#\n\x13InvokeActorResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"a\n\x17GetActorReminderRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torID\x12\x0c\n\x04name\x18\x03 \x01(\t\"\xdf\x01\n\x18GetActorReminderResponse\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torID\x12\x1e\n\x08\x64ue_time\x18\x04 \x01(\tH\x00R\x07\x64ueTime\x88\x01\x01\x12\x13\n\x06period\x18\x05 \x01(\tH\x01\x88\x01\x01\x12\"\n\x04\x64\x61ta\x18\x06 \x01(\x0b\x32\x14.google.protobuf.Any\x12\x10\n\x03ttl\x18\x07 \x01(\tH\x02\x88\x01\x01\x42\x0b\n\t_due_timeB\t\n\x07_periodB\x06\n\x04_ttl\"g\n\x19ListActorRemindersRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x1e\n\x08\x61\x63tor_id\x18\x02 \x01(\tH\x00R\x07\x61\x63torId\x88\x01\x01\x42\x0b\n\t_actor_id\"Z\n\x1aListActorRemindersResponse\x12<\n\treminders\x18\x01 \x03(\x0b\x32).dapr.proto.runtime.v1.NamedActorReminder\"Z\n\x12NamedActorReminder\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x36\n\x08reminder\x18\x02 \x01(\x0b\x32$.dapr.proto.runtime.v1.ActorReminder\"\xd4\x01\n\rActorReminder\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\tR\x07\x61\x63torID\x12\x1e\n\x08\x64ue_time\x18\x04 \x01(\tH\x00R\x07\x64ueTime\x88\x01\x01\x12\x13\n\x06period\x18\x05 \x01(\tH\x01\x88\x01\x01\x12\"\n\x04\x64\x61ta\x18\x06 \x01(\x0b\x32\x14.google.protobuf.Any\x12\x10\n\x03ttl\x18\x07 \x01(\tH\x02\x88\x01\x01\x42\x0b\n\t_due_timeB\t\n\x07_periodB\x06\n\x04_ttl\"s\n%UnregisterActorRemindersByTypeRequest\x12\x1d\n\nactor_type\x18\x01 \x01(\tR\tactorType\x12\x1e\n\x08\x61\x63tor_id\x18\x02 \x01(\tH\x00R\x07\x61\x63torId\x88\x01\x01\x42\x0b\n\t_actor_id\"(\n&UnregisterActorRemindersByTypeResponseBi\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\xaa\x02\x1b\x44\x61pr.Client.Autogen.Grpc.v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dapr.proto.runtime.v1.actors_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\252\002\033Dapr.Client.Autogen.Grpc.v1'
  _globals['_GETACTORSTATERESPONSE_METADATAENTRY']._loaded_options = None
  _globals['_GETACTORSTATERESPONSE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_TRANSACTIONALACTORSTATEOPERATION_METADATAENTRY']._loaded_options = None
  _globals['_TRANSACTIONALACTORSTATEOPERATION_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_INVOKEACTORREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_INVOKEACTORREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_REGISTERACTORTIMERREQUEST']._serialized_start=124
  _globals['_REGISTERACTORTIMERREQUEST']._serialized_end=311
  _globals['_UNREGISTERACTORTIMERREQUEST']._serialized_start=313
  _globals['_UNREGISTERACTORTIMERREQUEST']._serialized_end=414
  _globals['_REGISTERACTORREMINDERREQUEST']._serialized_start=417
  _globals['_REGISTERACTORREMINDERREQUEST']._serialized_end=731
  _globals['_UNREGISTERACTORREMINDERREQUEST']._serialized_start=733
  _globals['_UNREGISTERACTORREMINDERREQUEST']._serialized_end=837
  _globals['_GETACTORSTATEREQUEST']._serialized_start=839
  _globals['_GETACTORSTATEREQUEST']._serialized_end=932
  _globals['_GETACTORSTATERESPONSE']._serialized_start=935
  _globals['_GETACTORSTATERESPONSE']._serialized_end=1099
  _globals['_GETACTORSTATERESPONSE_METADATAENTRY']._serialized_start=1052
  _globals['_GETACTORSTATERESPONSE_METADATAENTRY']._serialized_end=1099
  _globals['_EXECUTEACTORSTATETRANSACTIONREQUEST']._serialized_start=1102
  _globals['_EXECUTEACTORSTATETRANSACTIONREQUEST']._serialized_end=1274
  _globals['_TRANSACTIONALACTORSTATEOPERATION']._serialized_start=1277
  _globals['_TRANSACTIONALACTORSTATEOPERATION']._serialized_end=1522
  _globals['_TRANSACTIONALACTORSTATEOPERATION_METADATAENTRY']._serialized_start=1052
  _globals['_TRANSACTIONALACTORSTATEOPERATION_METADATAENTRY']._serialized_end=1099
  _globals['_INVOKEACTORREQUEST']._serialized_start=1525
  _globals['_INVOKEACTORREQUEST']._serialized_end=1757
  _globals['_INVOKEACTORREQUEST_METADATAENTRY']._serialized_start=1052
  _globals['_INVOKEACTORREQUEST_METADATAENTRY']._serialized_end=1099
  _globals['_INVOKEACTORRESPONSE']._serialized_start=1759
  _globals['_INVOKEACTORRESPONSE']._serialized_end=1794
  _globals['_GETACTORREMINDERREQUEST']._serialized_start=1796
  _globals['_GETACTORREMINDERREQUEST']._serialized_end=1893
  _globals['_GETACTORREMINDERRESPONSE']._serialized_start=1896
  _globals['_GETACTORREMINDERRESPONSE']._serialized_end=2119
  _globals['_LISTACTORREMINDERSREQUEST']._serialized_start=2121
  _globals['_LISTACTORREMINDERSREQUEST']._serialized_end=2224
  _globals['_LISTACTORREMINDERSRESPONSE']._serialized_start=2226
  _globals['_LISTACTORREMINDERSRESPONSE']._serialized_end=2316
  _globals['_NAMEDACTORREMINDER']._serialized_start=2318
  _globals['_NAMEDACTORREMINDER']._serialized_end=2408
  _globals['_ACTORREMINDER']._serialized_start=2411
  _globals['_ACTORREMINDER']._serialized_end=2623
  _globals['_UNREGISTERACTORREMINDERSBYTYPEREQUEST']._serialized_start=2625
  _globals['_UNREGISTERACTORREMINDERSBYTYPEREQUEST']._serialized_end=2740
  _globals['_UNREGISTERACTORREMINDERSBYTYPERESPONSE']._serialized_start=2742
  _globals['_UNREGISTERACTORREMINDERSBYTYPERESPONSE']._serialized_end=2782
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

"""
Copyright 2023 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import grpc

from contextlib import contextmanager
from typing import Optional
from dapr.conf import settings

from dapr.proto.common.v1 import common_pb2 as common_v1
import dapr.proto.runtime.v1 as api_v1
from dapr.proto.runtime.v1 import dapr_pb2_grpc as api_service_v1

from dapr.proto.runtime.v1 import appcallback_pb2 as appcallback_v1
from dapr.proto.runtime.v1 import appcallback_pb2_grpc as appcallback_service_v1


@contextmanager
def connect_dapr(port: Optional[int] = -1):
    if port == -1:
        port = settings.DAPR_GRPC_PORT
    channel = grpc.insecure_channel(f"127.0.0.1:{port}")
    stub = api_service_v1.DaprStub(channel)
    yield stub
    channel.close()


__all__ = [
    'connect_dapr',
    'common_v1',
    'api_v1',
    'appcallback_v1',
    'appcallback_service_v1',
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: dapr/proto/runtime/v1/pubsub.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'dapr/proto/runtime/v1/pubsub.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from dapr.proto.runtime.v1 import appcallback_pb2 as dapr_dot_proto_dot_runtime_dot_v1_dot_appcallback__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"dapr/proto/runtime/v1/pubsub.proto\x12\x15\x64\x61pr.proto.runtime.v1\x1a\'dapr/proto/runtime/v1/appcallback.proto\"\xdf\x01\n\x13PublishEventRequest\x12\x13\n\x0bpubsub_name\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x19\n\x11\x64\x61ta_content_type\x18\x04 \x01(\t\x12J\n\x08metadata\x18\x05 \x03(\x0b\x32\x38.dapr.proto.runtime.v1.PublishEventRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xf5\x01\n\x12\x42ulkPublishRequest\x12\x13\n\x0bpubsub_name\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\x12?\n\x07\x65ntries\x18\x03 \x03(\x0b\x32..dapr.proto.runtime.v1.BulkPublishRequestEntry\x12I\n\x08metadata\x18\x04 \x03(\x0b\x32\x37.dapr.proto.runtime.v1.BulkPublishRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xd1\x01\n\x17\x42ulkPublishRequestEntry\x12\x10\n\x08\x65ntry_id\x18\x01 \x01(\t\x12\r\n\x05\x65vent\x18\x02 \x01(\x0c\x12\x14\n\x0c\x63ontent_type\x18\x03 \x01(\t\x12N\n\x08metadata\x18\x04 \x03(\x0b\x32<.dapr.proto.runtime.v1.BulkPublishRequestEntry.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"c\n\x13\x42ulkPublishResponse\x12L\n\rfailedEntries\x18\x01 \x03(\x0b\x32\x35.dapr.proto.runtime.v1.BulkPublishResponseFailedEntry\"A\n\x1e\x42ulkPublishResponseFailedEntry\x12\x10\n\x08\x65ntry_id\x18\x01 \x01(\t\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"\x84\x02\n!SubscribeTopicEventsRequestAlpha1\x12Z\n\x0finitial_request\x18\x01 \x01(\x0b\x32?.dapr.proto.runtime.v1.SubscribeTopicEventsRequestInitialAlpha1H\x00\x12\\\n\x0f\x65vent_processed\x18\x02 \x01(\x0b\x32\x41.dapr.proto.runtime.v1.SubscribeTopicEventsRequestProcessedAlpha1H\x00\x42%\n#subscribe_topic_events_request_type\"\x96\x02\n(SubscribeTopicEventsRequestInitialAlpha1\x12\x13\n\x0bpubsub_name\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\x12_\n\x08metadata\x18\x03 \x03(\x0b\x32M.dapr.proto.runtime.v1.SubscribeTopicEventsRequestInitialAlpha1.MetadataEntry\x12\x1e\n\x11\x64\x65\x61\x64_letter_topic\x18\x04 \x01(\tH\x00\x88\x01\x01\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x14\n\x12_dead_letter_topic\"s\n*SubscribeTopicEventsRequestProcessedAlpha1\x12\n\n\x02id\x18\x01 \x01(\t\x12\x39\n\x06status\x18\x02 \x01(\x0b\x32).dapr.proto.runtime.v1.TopicEventResponse\"\xed\x01\n\"SubscribeTopicEventsResponseAlpha1\x12\\\n\x10initial_response\x18\x01 \x01(\x0b\x32@.dapr.proto.runtime.v1.SubscribeTopicEventsResponseInitialAlpha1H\x00\x12\x41\n\revent_message\x18\x02 \x01(\x0b\x32(.dapr.proto.runtime.v1.TopicEventRequestH\x00\x42&\n$subscribe_topic_events_response_type\"+\n)SubscribeTopicEventsResponseInitialAlpha1Bi\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\xaa\x02\x1b\x44\x61pr.Client.Autogen.Grpc.v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dapr.proto.runtime.v1.pubsub_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\nio.dapr.v1B\nDaprProtosZ1github.com/dapr/dapr/pkg/proto/runtime/v1;runtime\252\002\033Dapr.Client.Autogen.Grpc.v1'
  _globals['_PUBLISHEVENTREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_PUBLISHEVENTREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_BULKPUBLISHREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_BULKPUBLISHREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_BULKPUBLISHREQUESTENTRY_METADATAENTRY']._loaded_options = None
  _globals['_BULKPUBLISHREQUESTENTRY_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_SUBSCRIBETOPICEVENTSREQUESTINITIALALPHA1_METADATAENTRY']._loaded_options = None
  _globals['_SUBSCRIBETOPICEVENTSREQUESTINITIALALPHA1_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_PUBLISHEVENTREQUEST']._serialized_start=103
  _globals['_PUBLISHEVENTREQUEST']._serialized_end=326
  _globals['_PUBLISHEVENTREQUEST_METADATAENTRY']._serialized_start=279
  _globals['_PUBLISHEVENTREQUEST_METADATAENTRY']._serialized_end=326
  _globals['_BULKPUBLISHREQUEST']._serialized_start=329
  _globals['_BULKPUBLISHREQUEST']._serialized_end=574
  _globals['_BULKPUBLISHREQUEST_METADATAENTRY']._serialized_start=279
  _globals['_BULKPUBLISHREQUEST_METADATAENTRY']._serialized_end=326
  _globals['_BULKPUBLISHREQUESTENTRY']._serialized_start=577
  _globals['_BULKPUBLISHREQUESTENTRY']._serialized_end=786
  _globals['_BULKPUBLISHREQUESTENTRY_METADATAENTRY']._serialized_start=279
  _globals['_BULKPUBLISHREQUESTENTRY_METADATAENTRY']._serialized_end=326
  _globals['_BULKPUBLISHRESPONSE']._serialized_start=788
  _globals['_BULKPUBLISHRESPONSE']._serialized_end=887
  _globals['_BULKPUBLISHRESPONSEFAILEDENTRY']._serialized_start=889
  _globals['_BULKPUBLISHRESPONSEFAILEDENTRY']._serialized_end=954
  _globals['_SUBSCRIBETOPICEVENTSREQUESTALPHA1']._serialized_start=957
  _globals['_SUBSCRIBETOPICEVENTSREQUESTALPHA1']._serialized_end=1217
  _globals['_SUBSCRIBETOPICEVENTSREQUESTINITIALALPHA1']._serialized_start=1220
  _globals['_SUBSCRIBETOPICEVENTSREQUESTINITIALALPHA1']._serialized_end=1498
  _globals['_SUBSCRIBETOPICEVENTSREQUESTINITIALALPHA1_METADATAENTRY']._serialized_start=279
  _globals['_SUBSCRIBETOPICEVENTSREQUESTINITIALALPHA1_METADATAENTRY']._serialized_end=326
  _globals['_SUBSCRIBETOPICEVENTSREQUESTPROCESSEDALPHA1']._serialized_start=1500
  _globals['_SUBSCRIBETOPICEVENTSREQUESTPROCESSEDALPHA1']._serialized_end=1615
  _globals['_SUBSCRIBETOPICEVENTSRESPONSEALPHA1']._serialized_start=1618
  _globals['_SUBSCRIBETOPICEVENTSRESPONSEALPHA1']._serialized_end=1855
  _globals['_SUBSCRIBETOPICEVENTSRESPONSEINITIALALPHA1']._serialized_start=1857
  _globals['_SUBSCRIBETOPICEVENTSRESPONSEINITIALALPHA1']._serialized_end=1900
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

"""
Copyright 2025 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import inspect
import random
import string
import types
from dataclasses import fields, is_dataclass
from enum import Enum
from typing import (
    Any,
    Callable,
    Dict,
    List,
    Literal,
    Mapping,
    Optional,
    Sequence,
    Union,
    cast,
    get_args,
    get_origin,
    get_type_hints,
)

from dapr.conf import settings

# Make mypy happy. Runtime handle: real class on 3.10+, else None.
# TODO: Python 3.9 is about to be end-of-life, so we can drop this at some point next year (2026)
UnionType: Any = getattr(types, 'UnionType', None)

# duplicated from conversation to avoid circular import
Params = Union[Mapping[str, Any], Sequence[Any], None]

"""
Tool Calling Helpers for Dapr Conversation API.

This module provides function-to-JSON-schema helpers that automatically
convert typed Python functions to tools for the Conversation API.

These makes it easy to create tools for the Conversation API without
having to manually define the JSON schema for each tool.
"""


def _python_type_to_json_schema(python_type: Any, field_name: str = '') -> Dict[str, Any]:
    """Convert a Python type hint to JSON schema format.

    Args:
        python_type: The Python type to convert
        field_name: The name of the field (for better error messages)

    Returns:
        Dict representing the JSON schema for this type

    Examples:
        >>> _python_type_to_json_schema(str)
        {"type": "string"}
        >>> _python_type_to_json_schema(Optional[int])
        {"type": "integer"}
        >>> _python_type_to_json_schema(List[str])
        {"type": "array", "items": {"type": "string"}}
    """
    # Handle None type
    if python_type is type(None):
        return {'type': 'null'}

    # Get the origin type for generic types (List, Dict, Union, etc.)
    origin = get_origin(python_type)
    args = get_args(python_type)

    # Handle Union types (including Optional which is Union[T, None])
    if origin is Union:
        # Check if this is Optional[T] (Union[T, None])
        non_none_args = [arg for arg in args if arg is not type(None)]
        if len(non_none_args) == 1 and type(None) in args:
            # This is Optional[T], convert T
            return _python_type_to_json_schema(non_none_args[0], field_name)
        else:
            # This is a true Union, use anyOf
            return {'anyOf': [_python_type_to_json_schema(arg, field_name) for arg in args]}

    # Handle Literal types -> map to enum
    if origin is Literal:
        # Normalize literal values (convert Enum members to their value)
        literal_values: List[Any] = []
        for val in args:
            try:
                from enum import Enum as _Enum

                if isinstance(val, _Enum):
                    literal_values.append(val.value)
                else:
                    literal_values.append(val)
            except Exception:
                literal_values.append(val)

        # Determine JSON Schema primitive types for provided literals
        def _json_primitive_type(v: Any) -> str:
            if v is None:
                return 'null'
            if isinstance(v, bool):
                return 'boolean'
            if isinstance(v, int) and not isinstance(v, bool):
                return 'integer'
            if isinstance(v, float):
                return 'number'
            if isinstance(v, (bytes, bytearray)):
                return 'string'
            if isinstance(v, str):
                return 'string'
            # Fallback: let enum carry through without explicit type
            return 'string'

        types = {_json_primitive_type(v) for v in literal_values}
        schema: Dict[str, Any] = {'enum': literal_values}
        # If all non-null literals share same type, include it
        non_null_types = {t for t in types if t != 'null'}
        if len(non_null_types) == 1 and (len(types) == 1 or len(types) == 2 and 'null' in types):
            only_type = next(iter(non_null_types)) if non_null_types else 'null'
            if only_type == 'string' and any(
                isinstance(v, (bytes, bytearray)) for v in literal_values
            ):
                schema['type'] = 'string'
                # Note: bytes literals represented as raw bytes are unusual; keeping enum as-is.
            else:
                schema['type'] = only_type
        elif types == {'null'}:
            schema['type'] = 'null'
        return schema

    # Handle List types
    if origin is list or python_type is list:
        if args:
            return {
                'type': 'array',
                'items': _python_type_to_json_schema(args[0], f'{field_name}[]'),
            }
        else:
            return {'type': 'array'}

    # Handle Dict types
    if origin is dict or python_type is dict:
        schema = {'type': 'object'}
        if args and len(args) == 2:
            # Dict[str, ValueType] - add additionalProperties
            key_type, value_type = args
            if key_type is str:
                schema['additionalProperties'] = _python_type_to_json_schema(
                    value_type, f'{field_name}.*'
                )
        return schema

    # Handle basic types
    if python_type is str:
        return {'type': 'string'}
    elif python_type is int:
        return {'type': 'integer'}
    elif python_type is float:
        return {'type': 'number'}
    elif python_type is bool:
        return {'type': 'boolean'}
    elif python_type is bytes:
        return {'type': 'string', 'format': 'byte'}

    # Handle Enum types
    if inspect.isclass(python_type) and issubclass(python_type, Enum):
        try:
            members = list(python_type)
        except Exception:
            members = []
        count = len(members)
        # If enum is small enough, include full enum list (current behavior)
        if count <= settings.DAPR_CONVERSATION_TOOLS_MAX_ENUM_ITEMS:
            return {'type': 'string', 'enum': [item.value for item in members]}
        # Large enum handling
        if settings.DAPR_CONVERSATION_TOOLS_LARGE_ENUM_BEHAVIOR == 'error':
            raise ValueError(
                f"Enum '{getattr(python_type, '__name__', str(python_type))}' has {count} members, "
                f'exceeding DAPR_CONVERSATION_MAX_ENUM_ITEMS={settings.DAPR_CONVERSATION_TOOLS_MAX_ENUM_ITEMS}. '
                f'Either reduce the enum size or set DAPR_CONVERSATION_LARGE_ENUM_BEHAVIOR=string to allow compact schema.'
            )
        # Default behavior: compact schema as a string with helpful context and a few examples
        example_values = [item.value for item in members[:5]] if members else []
        desc = (
            f'{getattr(python_type, "__name__", "Enum")} (enum with {count} values). '
            f'Provide a valid value. Schema compacted to avoid oversized enum listing.'
        )
        schema = {'type': 'string', 'description': desc}
        if example_values:
            schema['examples'] = example_values
        return schema

    # Handle Pydantic models (if available)
    if hasattr(python_type, 'model_json_schema'):
        try:
            return python_type.model_json_schema()
        except Exception:
            pass
    elif hasattr(python_type, 'schema'):
        try:
            return python_type.schema()
        except Exception:
            pass

    # Handle dataclasses
    if is_dataclass(python_type):
        from dataclasses import MISSING

        dataclass_schema: Dict[str, Any] = {'type': 'object', 'properties': {}, 'required': []}

        for field in fields(python_type):
            field_schema = _python_type_to_json_schema(field.type, field.name)
            dataclass_schema['properties'][field.name] = field_schema

            # Check if field has no default (required) - use MISSING for dataclasses
            if field.default is MISSING:
                dataclass_schema['required'].append(field.name)

        return dataclass_schema

    # Handle plain classes (non-dataclass) using __init__ signature and annotations
    if inspect.isclass(python_type) and python_type is not Any:
        try:
            # Gather type hints from __init__ if available; fall back to class annotations
            init = getattr(python_type, '__init__', None)
            init_hints = get_type_hints(init) if init else {}
            class_hints = get_type_hints(python_type)
        except Exception:
            init_hints = {}
            class_hints = {}

        # Build properties from __init__ parameters (excluding self)
        properties: Dict[str, Any] = {}
        required: List[str] = []

        try:
            sig = inspect.signature(python_type)
        except Exception:
            sig = None  # type: ignore

        check_slots = True
        if sig is not None:
            check_slots = False
            for pname, param in sig.parameters.items():
                if pname == 'self':
                    continue
                # Determine type for this parameter
                ptype = init_hints.get(pname) or class_hints.get(pname) or Any
                properties[pname] = _python_type_to_json_schema(ptype, pname)
                # Required if no default provided and not VAR_KEYWORD/POSITIONAL
                if param.default is inspect._empty and param.kind in (
                    inspect.Parameter.POSITIONAL_ONLY,
                    inspect.Parameter.POSITIONAL_OR_KEYWORD,
                    inspect.Parameter.KEYWORD_ONLY,
                ):
                    required.append(pname)
            else:
                check_slots = True
        if check_slots:
            # Fall back to __slots__ if present
            slots = getattr(python_type, '__slots__', None)
            if isinstance(slots, (list, tuple)):
                for pname in slots:
                    ptype = class_hints.get(pname, Any)
                    properties[pname] = _python_type_to_json_schema(ptype, pname)
                    if not (get_origin(ptype) is Union and type(None) in get_args(ptype)):
                        required.append(pname)
            else:  # use class_hints
                for pname, ptype in class_hints.items():
                    properties[pname] = _python_type_to_json_schema(ptype, pname)
                    if not (get_origin(ptype) is Union and type(None) in get_args(ptype)):
                        required.append(pname)

        # If we found nothing, return a generic object
        if not properties:
            return {'type': 'object'}

        schema = {'type': 'object', 'properties': properties}
        if required:
            schema['required'] = required
        return schema

    # Fallback for unknown/unsupported types
    raise TypeError(
        f"Unsupported type in JSON schema conversion for field '{field_name}': {python_type}. "
        f'Please use supported typing annotations (e.g., str, int, float, bool, bytes, List[T], Dict[str, V], Union, Optional, Literal, Enum, dataclass, or plain classes).'
        f'You can report this issue for future support of this type. You can always create the json schema manually.'
    )


def _extract_docstring_args(func) -> Dict[str, str]:
    """Extract parameter descriptions from function docstring.

    Supports Google-style, NumPy-style, and Sphinx-style docstrings.

    Args:
        func: The function to analyze

    Returns:
        Dict mapping parameter names to their descriptions

    Raises:
        ValueError: If docstring contains parameter info but doesn't match supported formats
    """
    docstring = inspect.getdoc(func)
    if not docstring:
        return {}

    param_descriptions = {}
    lines = docstring.split('\n')

    # First, try to extract Sphinx-style parameters (:param name: description)
    param_descriptions.update(_extract_sphinx_params(lines))

    # If no Sphinx-style params found, try Google/NumPy style
    if not param_descriptions:
        param_descriptions.update(_extract_google_numpy_params(lines))

    # If still no parameters found, check if docstring might have parameter info
    # in an unsupported format
    if not param_descriptions and _has_potential_param_info(lines):
        func_name = getattr(func, '__name__', 'unknown')
        import warnings

        warnings.warn(
            f"Function '{func_name}' has a docstring that appears to contain parameter "
            f"information, but it doesn't match any supported format (Google, NumPy, or Sphinx style). "
            f'Consider reformatting the docstring to use one of the supported styles for '
            f'automatic parameter extraction or create the tool manually.',
            UserWarning,
            stacklevel=2,
        )

    return param_descriptions


def _has_potential_param_info(lines: List[str]) -> bool:
    """Check if docstring lines might contain parameter information in unsupported format.

    This is a heuristic to detect when a docstring might have parameter info
    but doesn't match our supported formats (Google, NumPy, Sphinx).
    """
    text = ' '.join(line.strip().lower() for line in lines)

    # Look for specific parameter documentation patterns that suggest
    # an attempt to document parameters in an unsupported format
    import re

    # Look for informal parameter descriptions like:
    # "The filename parameter should be..." or "parameter_name is used for..."
    has_param_descriptions = bool(
        re.search(r'the\s+\w+\s+parameter\s+(should|is|controls|specifies)', text)
    )

    # Look for patterns where parameters are mentioned with descriptions
    # "filename parameter", "mode parameter", etc.
    has_param_mentions = bool(
        re.search(r'\w+\s+parameter\s+(should|is|controls|specifies|contains)', text)
    )

    # Look for informal patterns like "takes param1 which is", "param2 is an integer"
    has_informal_param_descriptions = bool(
        re.search(r'takes\s+\w+\s+which\s+(is|are)', text)
        or re.search(r'\w+\s+(is|are)\s+(a|an)\s+\w+\s+(input|argument)', text)
    )

    # Look for multiple parameter mentions suggesting documentation attempt
    param_count = text.count(' parameter ')
    has_multiple_param_mentions = param_count >= 2

    # Exclude common phrases that don't indicate parameter documentation attempts
    exclude_phrases = [
        'no parameters',
        'without parameters',
        'parameters documented',
        'parameter information',
        'parameter extraction',
        'function parameter',
        'optional parameter',
        'required parameter',  # These are often in general descriptions
    ]
    has_excluded_phrases = any(phrase in text for phrase in exclude_phrases)

    return (
        has_param_descriptions
        or has_param_mentions
        or has_informal_param_descriptions
        or has_multiple_param_mentions
    ) and not has_excluded_phrases


def _extract_sphinx_params(lines: List[str]) -> Dict[str, str]:
    """Extract parameters from Sphinx-style docstring.

    Looks for patterns like:
    :param name: description
    :parameter name: description
    """
    import re

    param_descriptions = {}

    for original_line in lines:
        line = original_line.strip()

        # Match Sphinx-style parameter documentation
        # Patterns: :param name: description or :parameter name: description
        param_match = re.match(r':param(?:eter)?\s+(\w+)\s*:\s*(.*)', line)
        if param_match:
            param_name = param_match.group(1)
            description = param_match.group(2).strip()
            param_descriptions[param_name] = description
            continue

        # Handle multi-line descriptions for Sphinx style
        # If line is indented and we have existing params, it might be a continuation
        if (
            original_line.startswith('    ') or original_line.startswith('\t')
        ) and param_descriptions:
            # Check if this could be a continuation of the last parameter
            last_param = list(param_descriptions.keys())[-1]
            # Don't treat section headers or other directive-like content as continuations
            # Also don't treat content that looks like parameter definitions from other styles
            if (
                param_descriptions[last_param]
                and not any(
                    line.startswith(prefix) for prefix in [':param', ':type', ':return', ':raises']
                )
                and not line.lower().endswith(':')
                and not line.lower() in ('args', 'arguments', 'parameters', 'params')
                and ':' not in line
            ):  # Avoid treating "param1: description" as continuation
                param_descriptions[last_param] += ' ' + line.strip()

    return param_descriptions


def _extract_google_numpy_params(lines: List[str]) -> Dict[str, str]:
    """Extract parameters from Google/NumPy-style docstring."""
    param_descriptions = {}
    in_args_section = False
    current_param = None

    for i, original_line in enumerate(lines):
        line = original_line.strip()

        # Detect Args/Parameters section
        if line.lower() in ('args:', 'arguments:', 'parameters:', 'params:'):
            in_args_section = True
            continue

        # Handle NumPy style section headers with dashes
        if line.lower() in ('parameters', 'arguments') and in_args_section is False:
            in_args_section = True
            continue

        # Skip NumPy-style separator lines (dashes) but also check if this signals section end
        if in_args_section and line and all(c in '-=' for c in line):
            # Check if next line starts a new section
            next_line_idx = i + 1
            if next_line_idx < len(lines):
                next_line = lines[next_line_idx].strip().lower()
                if next_line in (
                    'returns',
                    'return',
                    'yields',
                    'yield',
                    'raises',
                    'raise',
                    'notes',
                    'note',
                    'examples',
                    'example',
                ):
                    in_args_section = False
            continue

        # Exit args section on new section
        if in_args_section and (line.endswith(':') and not line.startswith(' ')):
            in_args_section = False
            continue

        # Also exit on direct section headers without separators
        if in_args_section and line.lower() in (
            'returns',
            'return',
            'yields',
            'yield',
            'raises',
            'raise',
            'notes',
            'note',
            'examples',
            'example',
        ):
            in_args_section = False
            continue

        if in_args_section and line:
            # Look for parameter definitions (contains colon)
            if ':' in line:
                parts = line.split(':', 1)
                if len(parts) == 2:
                    param_name = parts[0].strip()
                    description = parts[1].strip()

                    # Handle type annotations like "param_name (type): description"
                    if '(' in param_name and ')' in param_name:
                        param_name = param_name.split('(')[0].strip()
                    # Handle NumPy style "param_name : type" format where description is on next line
                    if ' ' in param_name:
                        param_name = param_name.split()[0]

                    # Check if this looks like a real description vs just a type annotation
                    # For NumPy style: "param : type" vs Google style: "param: description"
                    # Type annotations are usually single words like "str", "int", "float"
                    # Descriptions have multiple words or punctuation
                    if description:
                        if description.replace(' ', '').isalnum() and len(description.split()) == 1:
                            # Likely just a type annotation (single alphanumeric word), wait for real description
                            param_descriptions[param_name] = ''
                        else:
                            # Contains multiple words or punctuation, likely a real description
                            param_descriptions[param_name] = description
                    else:
                        param_descriptions[param_name] = ''
                    current_param = param_name
            elif (
                current_param
                and (original_line.startswith('    ') or original_line.startswith('\t'))
                and in_args_section
            ):
                # Indented continuation line for current parameter (only if still in args section)
                if not param_descriptions[current_param]:
                    # First description line for this parameter (for cases where description is on next line)
                    param_descriptions[current_param] = line
                else:
                    # Additional description lines
                    param_descriptions[current_param] += ' ' + line

    return param_descriptions


def extract_docstring_summary(func) -> Optional[str]:
    """Extract only the summary from a function's docstring.

    Args:
        func: The function to extract the summary from

    Returns:
        The summary portion of the docstring, or None if no docstring exists
    """
    docstring = inspect.getdoc(func)
    if not docstring:
        return None

    lines = docstring.strip().split('\n')
    if not lines:
        return None

    # Extract all lines before the first section header
    summary_lines = []

    for line in lines:
        line = line.strip()

        # Skip empty lines
        if not line:
            continue

        # Check if this line starts a Google/NumPy-style section
        google_numpy_headers = (
            'args:',
            'arguments:',
            'parameters:',
            'params:',
            'returns:',
            'return:',
            'yields:',
            'yield:',
            'raises:',
            'raise:',
            'note:',
            'notes:',
            'example:',
            'examples:',
            'see also:',
            'references:',
            'attributes:',
        )
        if line.lower().endswith(':') and line.lower() in google_numpy_headers:
            break

        # Check if this line starts a Sphinx-style section
        # Look for patterns like :param name:, :returns:, :raises:, etc.
        import re

        sphinx_pattern = r'^:(?:param|parameter|type|returns?|return|yields?|yield|raises?|raise|note|notes|example|examples|see|seealso|references?|attributes?)(?:\s+\w+)?:'
        if re.match(sphinx_pattern, line.lower()):
            break

        summary_lines.append(line)

    return ' '.join(summary_lines) if summary_lines else None


def function_to_json_schema(
    func, name: Optional[str] = None, description: Optional[str] = None
) -> Dict[str, Any]:
    """Convert a Python function to a JSON schema for tool calling.
    All parameters without default values are set as required.

    Args:
        func: The Python function to convert
        name: Override the function name (defaults to func.__name__)
        description: Override the function description (defaults to first line of docstring)

    Returns:
        Complete JSON schema with properties and required fields

    Examples:
        >>> def get_weather(location: str, unit: str = "fahrenheit") -> str:
        ...     '''Get weather for a location.
        ...
        ...     Args:
        ...         location: The city name
        ...         unit: Temperature unit (celsius or fahrenheit)
        ...     '''
        ...     pass
        >>> schema = function_to_json_schema(get_weather)
        >>> schema["properties"]["location"]["type"]
        'string'
    """
    # Get function signature and type hints
    sig = inspect.signature(func)
    type_hints = get_type_hints(func)

    # Extract parameter descriptions from docstring
    param_descriptions = _extract_docstring_args(func)

    # Build JSON schema
    schema: Dict[str, Any] = {'type': 'object', 'properties': {}, 'required': []}

    for param_name, param in sig.parameters.items():
        # Skip *args and **kwargs
        if param.kind in (param.VAR_POSITIONAL, param.VAR_KEYWORD):
            continue

        # Get type hint
        param_type = type_hints.get(param_name, str)

        # Convert to JSON schema
        param_schema = _python_type_to_json_schema(param_type, param_name)

        # Add description if available
        if param_name in param_descriptions:
            param_schema['description'] = param_descriptions[param_name]

        schema['properties'][param_name] = param_schema

        # Check if parameter is required (no default value)
        if param.default is param.empty:
            schema['required'].append(param_name)

    return schema


def _generate_unique_tool_call_id():
    """Generate a unique ID for a tool call.  Mainly used if the LLM provider is not able to generate one itself."""
    return ''.join(random.choices(string.ascii_letters + string.digits, k=9))


def stringify_tool_output(value: Any) -> str:
    """Convert arbitrary tool return values into a serializable string.

    Rules:
    - If value is already a string, return as-is.
    - For bytes/bytearray, return a base64-encoded string with 'base64:' prefix (not JSON).
    - Otherwise, attempt to JSON-serialize the value and return the JSON string.
      Uses a conservative default encoder that supports only:
        * Enum -> enum.value (fallback to name)
        * dataclass -> asdict
      If JSON serialization still fails, fallback to str(value). If that fails, return '<unserializable>'.
    """
    import base64 as _b64
    import json as _json
    from dataclasses import asdict as _asdict

    if isinstance(value, str):
        return value

    # bytes/bytearray -> base64 string (raw, not JSON-quoted)
    if isinstance(value, (bytes, bytearray)):
        try:
            return 'base64:' + _b64.b64encode(bytes(value)).decode('ascii')
        except Exception:
            try:
                return str(value)
            except Exception:
                return '<unserializable>'

    def _default(o: Any):
        # Enum handling
        try:
            from enum import Enum as _Enum

            if isinstance(o, _Enum):
                try:
                    return o.value
                except Exception:
                    return getattr(o, 'name', str(o))
        except Exception:
            pass

        # dataclass handling
        try:
            if is_dataclass(o):
                # mypy: asdict expects a DataclassInstance; after the runtime guard, this cast is safe
                return _asdict(cast(Any, o))
        except Exception:
            pass

        # Plain Python objects with __dict__: return a dict filtered for non-callable attributes
        try:
            d = getattr(o, '__dict__', None)
            if isinstance(d, dict):
                return {k: v for k, v in d.items() if not callable(v)}
        except Exception:
            pass

        # Fallback: cause JSON to fail for unsupported types
        raise TypeError(f'Object of type {type(o).__name__} is not JSON serializable')

    try:
        return _json.dumps(value, default=_default, ensure_ascii=False)
    except Exception:
        try:
            # Last resort: convert to string
            return str(value)
        except Exception:
            return '<unserializable>'


# --- Tool Function Executor Backend

# --- Errors ----


class ToolError(RuntimeError): ...


class ToolNotFoundError(ToolError): ...


class ToolExecutionError(ToolError): ...


class ToolArgumentError(ToolError): ...


def _coerce_bool(value: Any) -> bool:
    if isinstance(value, bool):
        return value
    if isinstance(value, (int,)):
        return bool(value)
    if isinstance(value, str):
        v = value.strip().lower()
        if v in {'true', '1', 'yes', 'y', 'on'}:
            return True
        if v in {'false', '0', 'no', 'n', 'off'}:
            return False
    raise ValueError(f'Cannot coerce to bool: {value!r}')


def _coerce_scalar(value: Any, expected_type: Any) -> Any:
    # Basic scalar coercions
    if expected_type is str:
        return value if isinstance(value, str) else str(value)
    if expected_type is int:
        if isinstance(value, bool):  # avoid True->1 surprises
            return int(value)
        if isinstance(value, int):
            return value
        if isinstance(value, (float,)) and value.is_integer():
            return int(value)
        if isinstance(value, str):
            return int(value.strip())
        raise ValueError
    if expected_type is float:
        if isinstance(value, (int, float)):
            return float(value)
        if isinstance(value, str):
            return float(value.strip())
        raise ValueError
    if expected_type is bool:
        return _coerce_bool(value)
    return value


def _coerce_enum(value: Any, enum_type: Any) -> Any:
    # Accept enum instance, name, or value
    if isinstance(value, enum_type):
        return value
    try:
        # match by value
        for member in enum_type:
            if member.value == value:
                return member
        if isinstance(value, str):
            name = value.strip()
            try:
                return enum_type[name]
            except Exception:
                # try case-insensitive
                for member in enum_type:
                    if member.name.lower() == name.lower():
                        return member
    except Exception:
        pass
    raise ValueError(f'Cannot coerce {value!r} to {enum_type.__name__}')


def _coerce_literal(value: Any, lit_args: List[Any]) -> Any:
    # Try exact match first
    if value in lit_args:
        return value
    # Try string-to-number coercions if literal set is homogeneous numeric
    try_coerced: List[Any] = []
    for target in lit_args:
        try:
            if isinstance(target, int) and not isinstance(target, bool) and isinstance(value, str):
                try_coerced.append(int(value))
            elif isinstance(target, float) and isinstance(value, str):
                try_coerced.append(float(value))
            else:
                try_coerced.append(value)
        except Exception:
            try_coerced.append(value)
    for coerced in try_coerced:
        if coerced in lit_args:
            return coerced
    raise ValueError(f'{value!r} not in allowed literals {lit_args!r}')


def _is_union(t) -> bool:
    origin = get_origin(t)
    if origin is Union:
        return True
    return UnionType is not None and origin is UnionType


def _coerce_and_validate(value: Any, expected_type: Any) -> Any:
    args = get_args(expected_type)

    if expected_type is Any:
        raise TypeError('We cannot handle parameters with type Any')

    # Optional[T] -> Union[T, None]
    if _is_union(expected_type):
        # try each option
        last_err: Optional[Exception] = None
        for opt in args:
            if opt is type(None):
                if value is None:
                    return None
                continue
            try:
                return _coerce_and_validate(value, opt)
            except Exception as e:
                last_err = e
                continue
        raise ValueError(
            str(last_err) if last_err else f'Cannot coerce {value!r} to {expected_type}'
        )

    origin = get_origin(expected_type)

    # Literal
    if origin is Literal:
        return _coerce_literal(value, list(args))

    # List[T]
    if origin is list or expected_type is list:
        item_type = args[0] if args else Any
        if not isinstance(value, list):
            raise ValueError(f'Expected list, got {type(value).__name__}')
        return [_coerce_and_validate(v, item_type) for v in value]

    # Dict[K, V]
    if origin is dict or expected_type is dict:
        key_t = args[0] if len(args) > 0 else Any
        val_t = args[1] if len(args) > 1 else Any
        if not isinstance(value, dict):
            raise ValueError(f'Expected dict, got {type(value).__name__}')
        coerced: Dict[Any, Any] = {}
        for k, v in value.items():
            ck = _coerce_and_validate(k, key_t)
            cv = _coerce_and_validate(v, val_t)
            coerced[ck] = cv
        return coerced

    # Enums
    if inspect.isclass(expected_type) and issubclass(expected_type, Enum):
        return _coerce_enum(value, expected_type)

    # Dataclasses
    if inspect.isclass(expected_type) and is_dataclass(expected_type):
        if isinstance(value, expected_type) or value is None:
            return value
        raise ValueError(
            f'Expected {expected_type.__name__} dataclass instance, got {type(value).__name__}'
        )

    # Plain classes (construct from dict using __init__ where possible)
    if inspect.isclass(expected_type):
        if isinstance(value, expected_type):
            return value
        if isinstance(value, dict):
            try:
                sig = inspect.signature(expected_type)
            except Exception as e:
                raise ValueError(f'Cannot inspect constructor for {expected_type.__name__}: {e}')

            # type hints from __init__
            try:
                init_hints = get_type_hints(getattr(expected_type, '__init__', None))
            except Exception:
                init_hints = {}

            kwargs: Dict[str, Any] = {}
            missing: List[str] = []
            for pname, param in sig.parameters.items():
                if pname == 'self':
                    continue
                if pname in value:
                    et = init_hints.get(pname, Any)
                    kwargs[pname] = _coerce_and_validate(value[pname], et)
                else:
                    if param.default is inspect._empty and param.kind in (
                        inspect.Parameter.POSITIONAL_ONLY,
                        inspect.Parameter.POSITIONAL_OR_KEYWORD,
                        inspect.Parameter.KEYWORD_ONLY,
                    ):
                        missing.append(pname)
            if missing:
                raise ValueError(
                    f'Missing required constructor arg(s) for {expected_type.__name__}: {", ".join(missing)}'
                )
            try:
                return expected_type(**kwargs)
            except Exception as e:
                raise ValueError(f'Failed constructing {expected_type.__name__} with {kwargs}: {e}')
        # Not a dict or instance: fall through to isinstance check

    # Basic primitives
    try:
        return _coerce_scalar(value, expected_type)
    except Exception:
        # Fallback to isinstance check
        if expected_type is Any or isinstance(value, expected_type):
            return value
        raise ValueError(
            f'Expected {getattr(expected_type, "__name__", str(expected_type))}, got {type(value).__name__}'
        )


def bind_params_to_func(fn: Callable[..., Any], params: Params):
    """Bind parameters to a function in the correct order.

    Args:
        fn: The function to bind parameters to
        params: The parameters to bind

    Returns:
        The bound parameters
    """
    sig = inspect.signature(fn)
    if params is None:
        bound = sig.bind()
        bound.apply_defaults()
        return bound

    if isinstance(params, Mapping):
        bound = sig.bind_partial(**params)
        # missing required parameters
        missing = [
            p.name
            for p in sig.parameters.values()
            if p.default is inspect._empty
            and p.kind
            in (
                inspect.Parameter.POSITIONAL_ONLY,
                inspect.Parameter.POSITIONAL_OR_KEYWORD,
                inspect.Parameter.KEYWORD_ONLY,
            )
            and p.name not in bound.arguments
        ]
        if missing:
            raise ToolArgumentError(f'Missing required parameter(s): {", ".join(missing)}')
        # unexpected kwargs unless **kwargs present
        if not any(p.kind is inspect.Parameter.VAR_KEYWORD for p in sig.parameters.values()):
            extra = set(params) - set(sig.parameters)
            if extra:
                raise ToolArgumentError(f'Unexpected parameter(s): {", ".join(sorted(extra))}')
    elif isinstance(params, Sequence):
        bound = sig.bind(*params)
    else:
        raise ToolArgumentError('params must be a mapping (kwargs), sequence (positional), or None')

    bound.apply_defaults()

    # Coerce and validate according to type hints
    try:
        type_hints = get_type_hints(fn)
    except Exception:
        type_hints = {}
    for name, value in list(bound.arguments.items()):
        if name in type_hints:
            expected = type_hints[name]
            try:
                bound.arguments[name] = _coerce_and_validate(value, expected)
            except Exception as e:
                raise ToolArgumentError(
                    f"Invalid value for parameter '{name}': expected {getattr(get_origin(expected) or expected, '__name__', str(expected))}, got {type(value).__name__} ({value!r}). Details: {e}"
                ) from e

    return bound



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file

Copyright 2025 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import collections.abc
import dapr.proto.common.v1.common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class GetStateRequest(google.protobuf.message.Message):
    """GetStateRequest is the message to get key-value states from specific state store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    STORE_NAME_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    CONSISTENCY_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    store_name: builtins.str
    """The name of state store."""
    key: builtins.str
    """The key of the desired state"""
    consistency: dapr.proto.common.v1.common_pb2.StateOptions.StateConsistency.ValueType
    """The read consistency of the state store."""
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to state store components."""

    def __init__(
        self,
        *,
        store_name: builtins.str = ...,
        key: builtins.str = ...,
        consistency: dapr.proto.common.v1.common_pb2.StateOptions.StateConsistency.ValueType = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["consistency", b"consistency", "key", b"key", "metadata", b"metadata", "store_name", b"store_name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GetStateRequest: typing_extensions.TypeAlias = GetStateRequest

@typing.final
class GetBulkStateRequest(google.protobuf.message.Message):
    """GetBulkStateRequest is the message to get a list of key-value states from specific state store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    STORE_NAME_FIELD_NUMBER: builtins.int
    KEYS_FIELD_NUMBER: builtins.int
    PARALLELISM_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    store_name: builtins.str
    """The name of state store."""
    parallelism: builtins.int
    """The number of parallel operations executed on the state store for a get operation."""
    @property
    def keys(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The keys to get."""

    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to state store components."""

    def __init__(
        self,
        *,
        store_name: builtins.str = ...,
        keys: collections.abc.Iterable[builtins.str] | None = ...,
        parallelism: builtins.int = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["keys", b"keys", "metadata", b"metadata", "parallelism", b"parallelism", "store_name", b"store_name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GetBulkStateRequest: typing_extensions.TypeAlias = GetBulkStateRequest

@typing.final
class GetBulkStateResponse(google.protobuf.message.Message):
    """GetBulkStateResponse is the response conveying the list of state values."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ITEMS_FIELD_NUMBER: builtins.int
    @property
    def items(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___BulkStateItem]:
        """The list of items containing the keys to get values for."""

    def __init__(
        self,
        *,
        items: collections.abc.Iterable[Global___BulkStateItem] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["items", b"items"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GetBulkStateResponse: typing_extensions.TypeAlias = GetBulkStateResponse

@typing.final
class BulkStateItem(google.protobuf.message.Message):
    """BulkStateItem is the response item for a bulk get operation.
    Return values include the item key, data and etag.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    KEY_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    ETAG_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    key: builtins.str
    """state item key"""
    data: builtins.bytes
    """The byte array data"""
    etag: builtins.str
    """The entity tag which represents the specific version of data.
    ETag format is defined by the corresponding data store.
    """
    error: builtins.str
    """The error that was returned from the state store in case of a failed get operation."""
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to app."""

    def __init__(
        self,
        *,
        key: builtins.str = ...,
        data: builtins.bytes = ...,
        etag: builtins.str = ...,
        error: builtins.str = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["data", b"data", "error", b"error", "etag", b"etag", "key", b"key", "metadata", b"metadata"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___BulkStateItem: typing_extensions.TypeAlias = BulkStateItem

@typing.final
class GetStateResponse(google.protobuf.message.Message):
    """GetStateResponse is the response conveying the state value and etag."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    DATA_FIELD_NUMBER: builtins.int
    ETAG_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    """The byte array data"""
    etag: builtins.str
    """The entity tag which represents the specific version of data.
    ETag format is defined by the corresponding data store.
    """
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to app."""

    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
        etag: builtins.str = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["data", b"data", "etag", b"etag", "metadata", b"metadata"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GetStateResponse: typing_extensions.TypeAlias = GetStateResponse

@typing.final
class DeleteStateRequest(google.protobuf.message.Message):
    """DeleteStateRequest is the message to delete key-value states in the specific state store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    STORE_NAME_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    ETAG_FIELD_NUMBER: builtins.int
    OPTIONS_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    store_name: builtins.str
    """The name of state store."""
    key: builtins.str
    """The key of the desired state"""
    @property
    def etag(self) -> dapr.proto.common.v1.common_pb2.Etag:
        """The entity tag which represents the specific version of data.
        The exact ETag format is defined by the corresponding data store.
        """

    @property
    def options(self) -> dapr.proto.common.v1.common_pb2.StateOptions:
        """State operation options which includes concurrency/
        consistency/retry_policy.
        """

    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to state store components."""

    def __init__(
        self,
        *,
        store_name: builtins.str = ...,
        key: builtins.str = ...,
        etag: dapr.proto.common.v1.common_pb2.Etag | None = ...,
        options: dapr.proto.common.v1.common_pb2.StateOptions | None = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["etag", b"etag", "options", b"options"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["etag", b"etag", "key", b"key", "metadata", b"metadata", "options", b"options", "store_name", b"store_name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DeleteStateRequest: typing_extensions.TypeAlias = DeleteStateRequest

@typing.final
class DeleteBulkStateRequest(google.protobuf.message.Message):
    """DeleteBulkStateRequest is the message to delete a list of key-value states from specific state store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STORE_NAME_FIELD_NUMBER: builtins.int
    STATES_FIELD_NUMBER: builtins.int
    store_name: builtins.str
    """The name of state store."""
    @property
    def states(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[dapr.proto.common.v1.common_pb2.StateItem]:
        """The array of the state key values."""

    def __init__(
        self,
        *,
        store_name: builtins.str = ...,
        states: collections.abc.Iterable[dapr.proto.common.v1.common_pb2.StateItem] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["states", b"states", "store_name", b"store_name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DeleteBulkStateRequest: typing_extensions.TypeAlias = DeleteBulkStateRequest

@typing.final
class SaveStateRequest(google.protobuf.message.Message):
    """SaveStateRequest is the message to save multiple states into state store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STORE_NAME_FIELD_NUMBER: builtins.int
    STATES_FIELD_NUMBER: builtins.int
    store_name: builtins.str
    """The name of state store."""
    @property
    def states(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[dapr.proto.common.v1.common_pb2.StateItem]:
        """The array of the state key values."""

    def __init__(
        self,
        *,
        store_name: builtins.str = ...,
        states: collections.abc.Iterable[dapr.proto.common.v1.common_pb2.StateItem] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["states", b"states", "store_name", b"store_name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___SaveStateRequest: typing_extensions.TypeAlias = SaveStateRequest

@typing.final
class QueryStateRequest(google.protobuf.message.Message):
    """QueryStateRequest is the message to query state store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    STORE_NAME_FIELD_NUMBER: builtins.int
    QUERY_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    store_name: builtins.str
    """The name of state store."""
    query: builtins.str
    """The query in JSON format."""
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to state store components."""

    def __init__(
        self,
        *,
        store_name: builtins.str = ...,
        query: builtins.str = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["metadata", b"metadata", "query", b"query", "store_name", b"store_name"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___QueryStateRequest: typing_extensions.TypeAlias = QueryStateRequest

@typing.final
class QueryStateItem(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    ETAG_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    key: builtins.str
    """The object key."""
    data: builtins.bytes
    """The object value."""
    etag: builtins.str
    """The entity tag which represents the specific version of data.
    ETag format is defined by the corresponding data store.
    """
    error: builtins.str
    """The error message indicating an error in processing of the query result."""
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        data: builtins.bytes = ...,
        etag: builtins.str = ...,
        error: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["data", b"data", "error", b"error", "etag", b"etag", "key", b"key"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___QueryStateItem: typing_extensions.TypeAlias = QueryStateItem

@typing.final
class QueryStateResponse(google.protobuf.message.Message):
    """QueryStateResponse is the response conveying the query results."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    RESULTS_FIELD_NUMBER: builtins.int
    TOKEN_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    token: builtins.str
    """Pagination token."""
    @property
    def results(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___QueryStateItem]:
        """An array of query results."""

    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata which will be sent to app."""

    def __init__(
        self,
        *,
        results: collections.abc.Iterable[Global___QueryStateItem] | None = ...,
        token: builtins.str = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["metadata", b"metadata", "results", b"results", "token", b"token"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___QueryStateResponse: typing_extensions.TypeAlias = QueryStateResponse

@typing.final
class TransactionalStateOperation(google.protobuf.message.Message):
    """TransactionalStateOperation is the message to execute a specified operation with a key-value pair."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPERATIONTYPE_FIELD_NUMBER: builtins.int
    REQUEST_FIELD_NUMBER: builtins.int
    operationType: builtins.str
    """The type of operation to be executed"""
    @property
    def request(self) -> dapr.proto.common.v1.common_pb2.StateItem:
        """State values to be operated on"""

    def __init__(
        self,
        *,
        operationType: builtins.str = ...,
        request: dapr.proto.common.v1.common_pb2.StateItem | None = ...,
    ) -> None: ...
    _HasFieldArgType: typing_extensions.TypeAlias = typing.Literal["request", b"request"]
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["operationType", b"operationType", "request", b"request"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___TransactionalStateOperation: typing_extensions.TypeAlias = TransactionalStateOperation

@typing.final
class ExecuteStateTransactionRequest(google.protobuf.message.Message):
    """ExecuteStateTransactionRequest is the message to execute multiple operations on a specified store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["key", b"key", "value", b"value"]
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    STORENAME_FIELD_NUMBER: builtins.int
    OPERATIONS_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    storeName: builtins.str
    """Required. name of state store."""
    @property
    def operations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___TransactionalStateOperation]:
        """Required. transactional operation list."""

    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """The metadata used for transactional operations."""

    def __init__(
        self,
        *,
        storeName: builtins.str = ...,
        operations: collections.abc.Iterable[Global___TransactionalStateOperation] | None = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: typing_extensions.TypeAlias = typing.Literal["metadata", b"metadata", "operations", b"operations", "storeName", b"storeName"]
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ExecuteStateTransactionRequest: typing_extensions.TypeAlias = ExecuteStateTransactionRequest
